/*  File: analyser-thrust.cc
 *  Created by Zoltán Nagy on 14 January 2020.
 *  Modified by D. Soper on ....
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */


/*   local headers  */
#include "analyzer-thrust.h"



void UserThrust::initfunc()
{
  /*   Creating histograms to calculate the cross sections   */
  Hist1D::phys(1, "Thrust distribution", spacing<hist1d>::logarithmic(50, 1.0e-4, 0.5), true);
}




void UserThrust::userfunc(const shower_stage<epa>& hard,
                          const shower_stage<epa>& /*  afterthreshold  */,
                          const shower_stage<epa>& showered)
{
  /*   weights   */
  double wh = weight(hard);
  double ws = weight(showered);
  
  /*    Fill the thrust histograms   */
  double T = thrust(showered.state);

  Hist1D::physfill(1, (1.0-T)*ws, dirac{}, 1.0-T);
  Hist1D::normfill(1, wh);
}



double UserThrust::thrust(const event<epa>& p)
{
  energy<double> thrust = 0.0_GeV;
  std::size_t np = number_of<colored,fin>(p);
  
  /*    Loop over reference vectors orthogonal to two particles.    */
  for(std::size_t i = 1; i < np; ++i)
    for(std::size_t j = i + 1; j <= np; ++j)
    {
      auto part = threevector<energy<double>>{0.0_GeV,0.0_GeV,0.0_GeV};
      auto axis = cross(p[i].momentum, p[j].momentum)/1.0_GeV2;
      
      for(std::size_t k = 1; k <= np; ++k)
        if(k != i && k != j) {
          if(dot(p[k].momentum, axis) > 0.0_GeV) part += p[k].momentum;
          else part -= p[k].momentum;
        }
      
      /*    There are now four cases to be examined.   */
      if(auto trial = mag(part + p[i].momentum + p[j].momentum);  trial > thrust) thrust = trial;
      if(auto trial = mag(part + p[i].momentum - p[j].momentum);  trial > thrust) thrust = trial;
      if(auto trial = mag(part - p[i].momentum + p[j].momentum);  trial > thrust) thrust = trial;
      if(auto trial = mag(part - p[i].momentum - p[j].momentum);  trial > thrust) thrust = trial;
    }
  
  /*    return the normalized thrust    */
  return thrust/mag(p[-1].momentum + p[0].momentum);
}
  
