/*  File: analyser-thrust.cc
 *  Created by Zoltán Nagy on 14 January 2020.
 *  Modified by D. Soper on ....
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */


/*   local headers  */
#include "analyzer-y23.h"



void UserY23::initfunc()
{
  /*   Creating histograms to calculate the cross sections   */
  Hist1D::phys(1, "Cambridge y23 distribution", spacing<hist1d>::logarithmic(50, 1.0e-6, 0.5), true);
}




void UserY23::userfunc(const shower_stage<epa>& hard,
                          const shower_stage<epa>& /*  afterthreshold  */,
                          const shower_stage<epa>& showered)
{
  /*   weights   */
  double wh = weight(hard);
  double ws = weight(showered);
  
  /*    Fill the thrust histograms   */
  double y23 = cambridge_y23(showered.state);

  Hist1D::physfill(1, y23*ws, dirac{}, y23);
  Hist1D::normfill(1, wh);
}



double UserY23::cambridge_y23(event<epa> p)
{
  auto[lo,hi] = bounds_of<colored,fin>(p);
    
  /*   trivial case when we have only two partons   */
  if(hi == 2) return 0.0;
  
  
  /*   we have more than 2 partons   */
  energy_sqr<double> Q2 = mag2(p[-1].momentum + p[0].momentum);
  double y3 = 0.0;
  int imin = 0, jmin = 0;
  
  do {
    /*    find the minimum angular variable    */
    double vmin = 1e+100;
    
    for(int i = lo; i < hi; ++i)
      for(int j = i + 1; j <= hi; ++j)
        if(double vij = 2.0*(1.0-unit(p[i].momentum)*unit(p[j].momentum)); vij <= vmin)
        {
          vmin = vij;
          imin = i;
          jmin = j;
        }
    
    /*   resolution variable   */
    const energy<double> &Ei = p[imin].momentum[0], &Ej = p[jmin].momentum[0];
   
    if(double yij = (Ei < Ej ? Ei*Ei : Ej*Ej)*vmin/Q2; yij > y3)
      y3 = yij;
    
    
    /*   recombine  */
    p[imin].momentum +=  p[jmin].momentum;
    if(jmin != hi) p[jmin].momentum = p[hi].momentum;
    
  } while(--hi > 2);
  
  
  return y3;
}

